// #=============================================================================
// #
// #	ebsa285.S
// #
// #	EBSA285 Board specific code
// #
// #=============================================================================
// ####COPYRIGHTBEGIN####
// 
//  -------------------------------------------
//  Copyright(c) 1999-20000 hwang sungyeon
//  -------------------------------------------
// 
// ####COPYRIGHTEND####
// #=============================================================================
// ######DESCRIPTIONBEGIN####
// #
// # Author(s): 	hwang sungyeon
// # Contributors:	hwang sungyeon
// # Date:	1999-10-06
// # Purpose:	EBAS285 board init and misc assember codes.
// # Description: Yet.
// #		
// #        
// #
// #####DESCRIPTIONEND####
// #
// #=============================================================================

#include <pkgconf/hal.h>
#include <pkgconf/hal_arm.h>
#ifdef CYGPKG_KERNEL  // no CDL yet
#include <pkgconf/kernel.h>
#else
# undef CYGFUN_HAL_COMMON_KERNEL_SUPPORT
#endif
#include <cyg/hal/hal_platform_setup.h>

#ifdef CYGDBG_HAL_DEBUG_GDB_INCLUDE_STUBS
// The CDL should enforce this
#undef  CYGHWR_HAL_ARM_DUMP_EXCEPTIONS
#endif

#include "ebsa285.h"

	.file	"ebsa285.S"

	.text
	.align 4

//-------------------------------------------------------------
// init_x_bus
	.global init_x_bus
init_x_bus:	
	//	initialize the X bus, the we can see the UARTs and the LEDs
	ldr	r0, =SA_CTL_INIT
	ldr	r1, =CSR_BASE
	str	r0, [r1, #SA_CONTROL]
	
	// Write 0x1000:16db to the X-Bus Cycle/Arbiter regs.
	ldr	r0, =INIT_XBUS_CYCLE
	str	r0, [r1, #XBUS_CYCLE]
	
	// Write 0xFCFC:FCFC to the X-Bus I/O strobe regs.
	ldr r0, =INIT_XBUS_STROBE
	str	r0, [r1, #XBUS_STROBE]
	
	// Write zeros to the output of LPT2 ????
	ldr r0, =0x40011DE0
	ldr r1, =0
	str r1, [r1]
	
	mov pc, lr				// return

//---------------------------------------------------------------
// set_stub_ok_led
	.global set_romstub_ok_led
set_romstub_ok_led:

	ldr		r3, =0
	ldr		r4, =RED_LED
	bl	set_leds

	mov pc, lr

	
//======================================================================================
// General IRQ and mode change code for StrongARM(EBSA285)
//

//------------------------------------------------------------------------------------
// void disable_int(void)
	.global disable_int
disable_int:
	STMFD	sp!, {v1}
	MRS	v1, cpsr
	ORR	v1, v1, #NoIRQ
	MSR	cpsr, v1
	LDMFD	sp!, {v1}
	MOV	pc, lr

//-----------------------------------------------------------------------------------
// void enable_int(void)
	
	.global enable_int
enable_int:
	STMFD	sp!, {v1}
	MRS	v1, cpsr
	BIC	v1, v1, #NoIRQ
	MSR	cpsr, v1
	LDMFD	sp!, {v1}
	MOV	pc, lr

//------------------------------------------------------------------------------------
// int read_mode(void)
	.global read_mode
read_mode:
	MRS	a1, cpsr
	MOV	pc, lr

//-------------------------------------------------------------------------------------
// void write_mode(int mode)
	.global write_mode
write_mode:
	MSR	cpsr, a1		// change current psr
	MOV	pc, lr	

//=======================================================================================
// EBSA285 LED Control Module.
// SetLEDs
// set debug LEDs
//
// register usage
// r3 : off LED bits
// r4 : on LED bits
// r5 : CSR Base 
// r12: temp 

	.global set_leds
set_leds:
	mov		r2, #0				
	mov		r5, #0x40000000		
	add		r5, r5, #0x12000	
	and		r12, r2, r3
	eor		r12, r12, r4
	and		r2, r12, #7			
	eor		r12, r2, #7			
	strb	r12, [r5, #0]	
	
	mov pc, lr		// return
	
//=======================================================================================
// EBSA285 Low level Serial Device driver 
// reset_comm_port
// Initialize EBSA-285 UART
// Baud rate 9600

	.global reset_comm_port	
reset_comm_port:
	ldr		r2, =HOST_COMPORT
	ldr		r1, =0						// disable interrupts
	strb	r1, [r2, #AMBA_UARTCON]
	
	ldr		r1, =0						// set the baud rate to 9600
	strb	r1, [r2, #AMBA_M_UBRLCR]
	ldr		r1, =0x006
	strb	r1, [r2, #AMBA_L_UBRLCR]
	
	ldr		r1, =AMBA_H_UBRLCR_DSS_8;	// 8 bits, 1 stop, no parity, no fifo
	strb	r1, [r2, #AMBA_H_UBRLCR]
	
	ldr		r1, =AMBA_UARTCON_UE;		// enable the UART
	strb	r1, [r2, #AMBA_UARTCON]
	mov pc, lr		// return	

//----------------------------------------------------------------------------------
// get_byte
// Low level input function

	.global get_byte
get_byte:
				ldr		r0, =HOST_COMPORT
getbyteloop:	ldrb	r1, [r0, #AMBA_UARTFLG]
				ands	r1, r1, #AMBA_UARTFLG_RFE
				bne		getbyteloop
				ldrb	r0, [r0, #AMBA_UARTDR]		
				mov	pc, lr				// return
				
	.global	get_char
get_char:	
		STMFD	sp!, {r1-r2, lr}	// save working registers
		BL	get_byte			// returns char in r0
		LDMFD	sp!, {r1-r2, pc}	//restore registers & return
							
				
//----------------------------------------------------------------------------------
// put_byte
// Low level print function

	.global	put_byte
put_byte:
				ldr		r1, =HOST_COMPORT
putbyteloop:		
				ldrb	r2, [r1, #AMBA_UARTFLG]
				tst		r2, #AMBA_UARTFLG_TFF		// wait untill ready to queue character
				bne		putbyteloop
				strb	r0, [r1, #AMBA_UARTDR]				
				mov	pc, lr							// return

//----------------------------------------------------------------------------------		
// print_string
// High Level print string function

	.global print_string
print_string:
				mov		r4, lr
				mov		r5, r0
putachar:		ldrb	r0, [r5], #1
				tst		r0, #0
				moveq	pc, r4
				bl		put_byte
				b		putachar
				
				
//======================================================================================
// EBSA285 RAM INIT Module.
// init_ram
// Initialize memory on startup

	.global init_ram
init_ram:
	
0:
	// If memory is already active. need to disable refresh before
	// re-initialising
	mov		r3, #CSR_BASE
	mov		r2, #0
	str		r2, [r3, #DRAM_TIMING]
	
	// In case refresh is already happening, wait 50 clock cycles
	// until it expires
	mov		r1, #50
1:
	subs	r1, r1, #2
	bgt	1b					// until done
	
	// Read from each Mode Register array to enable precharge
	mov		r1, #MAX_SDRAM			// max no. of arrays = 4
	mov		r2, #DC21285_DRAM_A0MR	// base address of array 0 mode reg

1:	
	ldr		r4, [r2]				// get base address
	ldr		r4, [r2]				// just read, don't care about data
	add		r2, r2, #SD_OFFSET		// Next
	subs	r1, r1, #1
	bgt 1b							// until done
	
	// Now write to the Mode registers to set the operational mode of the
	// SDRAM. Must be done to all banks. even if not fitted, to stop the
	// 21285 forcing DQM High, which it does after reset.
	mov		r1, #MAX_SDRAM
	mov		r2, #DC21285_DRAM_A0MR
	
2:
	str		r2, [r2, #CAS_OFFSET]	// It's the ADDRESS which is important
	add		r2, r2, #SD_OFFSET		// Next
	subs	r1, r1, #1
	bgt		2b						// Until done
	
	
	// Setup Dram timing & DISABLE all arrays by default.
	
	// Setting the refresh interval to be minimum & enabling refresh.
	// After (upto) 8 refresh cycles the memory will be ready to use, each
	// refresh cycle is 32 instruction cycles.
	// R3 -> CSR_BASE
	ldr 	r2, =INITIAL_TIMING 		// minimum refresh timing
	str 	r2, [r3, #DRAM_TIMING] 		// Register offset 0x10C 
	mov 	r1, #(MAX_SDRAM << 2)
	mov 	r2, #0 						// Disable array & move to address 0 
	
3:
	str		r2, [r3, r1] 				// Registers are after timing register
	subs	r1, r1, #4
	bgt		3b							// Until done
	// Wait around until memory is active 

	mov		r1, #REFRESH_WAIT			// Number of refresh cycles to wait
4:
	mov		r2, #REFRESH_COUNT			// Length of each cycle 
5:
	subs	r2, r2, #2					// 2 instructions in this loop
	bgt		5b 							// Until cycle complete
	subs	r1, r1, #1 					// Another refresh cycle gone.. 
	bgt		4b							// Until done
	
	// At this point we could dynamically look at what sizes of
	// what sorts of DRAM arrays are fitted and then initialise them
	// accordingly. For the moment we assume that only the standard
	// single 2-array DIMM containing 2 arrays of 8 MB each is
	// fitted, making 16 MB in total.
	
	mov		r1, #DRAM_BASE 				// Start of RAM 
	mov		r3, #CSR_BASE
	ldr		r2, =SETUP_TIMING
	str		r2, [r3, #DRAM_TIMING]		// Enable 1st array with
	orr		r2, r1, #EBSA_BASE_MEM
	str		r2, [r3, #DRAM_ADDR_SIZE_0]	// Size & base location
	add		r1, r1, #EBSA_BASE_SIZE
	orr		r2, r1, #EBSA_BASE_MEM
	str		r2, [r3, #DRAM_ADDR_SIZE_1]	// Enable second array 

	
	mov	pc, lr				// return	

//==========================================================================================
// Cache Control Unit.

//-------------------------------------------------------------------------------------------
// write_cache_mode
// write the mmu state to the Co-processor

	.global write_cache_mode
write_cache_mode:
	stmfd	sp!,{a1, v1-v2, lr}		// Save registers
	stmfd	sp!,{a1}				//  save the new mode
	
	 //switch into SVC return SPSR in a1
	 //FIXME

	ldmfd	sp!,{v1}
  	mcr p15, 0, v1, c1, c0 ,0

	// Make sure the pipeline is clear of any cached entries
	nop
	nop
	nop

	// Switch back to the mode we started in (restores interrupts, if any)
    // FIXME
    	
	ldmfd	sp!,{a1, v1-v2, pc}		// restore registers and return

//-------------------------------------------------------------------------------------------
	.global read_cache_mode
read_cache_mode:
    mrc p15, 0, a1, c1, c0 ,0
	mov pc, lr						// return	
	
//-------------------------------------------------------------------------------------------
	.global disable_icache
disable_icache:
		stmfd	sp!, {r1-r2, lr}	// save working registers
		//FIXME
		ldmfd	sp!, {r1-r2, pc}	//restore registers & return

//-------------------------------------------------------------------------------------------
	.global disanle_dcache
disanle_dcache:
		stmfd	sp!,{a1, v1-v3, lr}	// Save registers
		//FIXME
		ldmfd	sp!,{a1, v1-v3, pc}	// restore registers and return

//-------------------------------------------------------------------------------------------
// R0 points to the start of a 16384 byte region of readable data used
// Return:
// R0, R1, R2 trashed
// Data Cache is clean	
	.global dcache_sync
dcache_sync:
		stmfd	sp!,{a1, v1-v3, lr}	// Save registers

		mov	r1, #MMU_INIT_VALUE								
		mcr	15, 0, r1, c7, c6, 0							

		ldmfd	sp!,{a1, v1-v3, pc}		// restore registers and return

	
	.global dcache_invalidate_all
dcache_invalidate_all:
		stmfd	sp!,{a1, v1-v3, lr}	// Save registers
			
		mov	r1, #MMU_INIT_VALUE								
		mcr	15, 0, r1, c7, c6, 0						
		
		ldmfd	sp!,{a1, v1-v3, pc}	// restore registers and return

//-------------------------------------------------------------------------------------------	
	.global enable_icache
enable_icache:
		stmfd	sp!,{a1, v1-v3, lr}	// Save registers
		//FIXME
		ldmfd	sp!,{a1, v1-v3, pc}	// restore registers and return
	

//-------------------------------------------------------------------------------------------
	.global enable_dcache
enable_dcache:
		stmfd	sp!,{a1, v1-v3, lr}	// Save registers
        // FIXME
		ldmfd   sp!,{a1, v1-v3, pc}	// restore registers and return
