#ifndef	include_ebsa285_h
#define	include_ebsa285_h

/*=============================================================================
//
//      ebsa285.h
//
//      HAL Support for EBSA285 
//
//=============================================================================
//####COPYRIGHTBEGIN####
//
//
// -------------------------------------------
//
//####COPYRIGHTEND####
//=============================================================================
//#####DESCRIPTIONBEGIN####
//
//
//####DESCRIPTIONEND####
//
//===========================================================================*/

#define ECOS_EBSA285

#define BUS		BUS_TYPE_PCI


/* bits in the PSR */

#define	V_BIT		(1<<28)
#define	C_BIT		(1<<29)
#define	Z_BIT		(1<<30)
#define	N_BIT		(1<<31)
#define I_BIT		(1<<7)
#define F_BIT		(1<<6)
#define	T_BIT		(1<<5)


/* mode bits */

#define MODE_SYSTEM32	0x1F
#define MODE_UNDEF32	0x1B
#define	MODE_ABORT32	0x17
#define MODE_SVC32		0x13
#define MODE_IRQ32		0x12
#define MODE_FIQ32		0x11
#define MODE_USER32		0x10


/* masks for getting bits from PSR */

#define MASK_MODE		0x0000003F
#define	MASK_32MODE		0x0000001F
#define	MASK_SUBMODE	0x0000000F
#define MASK_INT		0x000000C0
#define	MASK_CC			0xF0000000


/* shifts to access bits in the PSR */

#define INT_MASK_SHIFT	6
 // Local-to-Bus memory address constants:
 // the local memory address always appears at 0 locally;
 // it is not dual ported.
 //

#define LOCAL_MEM_LOCAL_ADRS    0x00000000     
#define LOCAL_MEM_BUS_ADRS      0x00000000    

 	
//----------------------------------------------------------------------------
// interrupt control stuff
// Note: FIQ is not handled within VxWorks so this is just IRQ


#define IC_BASE 	0x42000180
#define IRQ_STATUS	(IC_BASE+0x000) 	// Read 
#define IRQ_RAW_STATUS	(IC_BASE+0x004) // Read 
#define IRQ_ENABLE	(IC_BASE+0x008) 	// Read/Write 
#define IRQ_DISABLE	(IC_BASE+0x00C) 	// Write
#define IRQ_SOFT	(IC_BASE+0x010) 	// Write
#define FIQ_STATUS	(IC_BASE+0x100) 	// Read 
#define FIQ_RAW_STATUS	(IC_BASE+0x104) // Read 
#define FIQ_ENABLE	(IC_BASE+0x108) 	// Read/Write
#define FIQ_DISABLE	(IC_BASE+0x10C) 	// Write

#define AMBA_INT_NUM_LEVELS	32
#define AMBA_INT_CSR_PEND	IRQ_STATUS
#define AMBA_INT_CSR_ENB	IRQ_ENABLE
#define AMBA_INT_CSR_DIS	IRQ_DISABLE
#define AMBA_INT_CSR_MASK	((1 << INT_LVL_TIMER_2)  | \
				 (1 << INT_LVL_TIMER_3)  | \
				 (1 << INT_LVL_IRQ_IN_0) | \
				 (1 << INT_LVL_IRQ_IN_1) | \
				 (1 << INT_LVL_IRQ_IN_3) | \
				 (1 << INT_LVL_PCI)	 | \
				 (1 << INT_LVL_AMBARX)   | \
				 (1 << INT_LVL_AMBATX))

//---------------------------------------------------------------------------------
// interrupt levels 

#define INT_LVL_SOFT		1	/// soft interrupt 
#define INT_LVL_AMBARX		2	// AMBA serial port Rx
#define INT_LVL_AMBATX		3	// AMBA serial port Tx
#define INT_LVL_TIMER_1 	4	// timer 1 
#define INT_LVL_TIMER_2 	5	// timer 2 
#define INT_LVL_TIMER_3 	6	// timer 3 
#define INT_LVL_TIMER_4 	7	// timer 4 
#define	INT_LVL_IRQ_IN_0	8	// external to 21285 
#define	INT_LVL_IRQ_IN_1	9	// external to 21285 
#define	INT_LVL_IRQ_IN_2	10	// external to 21285 
#define	INT_LVL_IRQ_IN_3	11	// external to 21285 
#define INT_LVL_SPIO		12	// Super I/O serial ports
#define	INT_LVL_PCI			18	// PCI external interrupts

//---------------------------------------------------------------------------------
// interrupt vectors

#define INT_VEC_SOFT	 IVEC_TO_INUM(INT_LVL_SOFT)		// soft interrupt 
#define INT_VEC_AMBARX	 IVEC_TO_INUM(INT_LVL_AMBARX)	// AMBA serial port Rx
#define INT_VEC_AMBATX	 IVEC_TO_INUM(INT_LVL_AMBATX)	// AMBA serial port Tx
#define INT_VEC_TIMER_1  IVEC_TO_INUM(INT_LVL_TIMER_1)	// timer 1 
#define INT_VEC_TIMER_2  IVEC_TO_INUM(INT_LVL_TIMER_2)	// timer 2 
#define INT_VEC_TIMER_3  IVEC_TO_INUM(INT_LVL_TIMER_3)	// timer 3 
#define INT_VEC_TIMER_4  IVEC_TO_INUM(INT_LVL_TIMER_4)	// timer 4 
#define	INT_VEC_IRQ_IN_0 IVEC_TO_INUM(INT_LVL_IRQ_IN_0)	// external to 21285 
#define	INT_VEC_IRQ_IN_1 IVEC_TO_INUM(INT_LVL_IRQ_IN_1)	// external to 21285 
#define	INT_VEC_IRQ_IN_2 IVEC_TO_INUM(INT_LVL_IRQ_IN_2)	// external to 21285 
#define	INT_VEC_IRQ_IN_3 IVEC_TO_INUM(INT_LVL_IRQ_IN_3)	// external to 21285 
#define	INT_VEC_SPIO	 IVEC_TO_INUM(INT_LVL_SPIO)		// Super I/O serial 
#define INT_VEC_PCI	 IVEC_TO_INUM(INT_LVL_PCI)			// PCI external ints 

//----------------------------------------------------------------------------------
// miscellaneous EBSA-285 definitions
// 

// LED definitions 
#define LED_BANK						0x40012000

#define SA_LED_ON                       0
#define SA_LED_OFF                      1

#define RED_LED                         0x04
#define GREEN_LED                       0x02
#define YELLOW_LED                      0x01
#define ALL_LEDS                        0x07
#define SA_NUM_OF_LEDS                  3

#define SA_LED_MASKS   { 0, YELLOW_LED, GREEN_LED, RED_LED }
#define SA_LED_OFFSETS { 0, (void *)LED_BANK, (void *)LED_BANK, (void *)LED_BANK }

#ifndef LED_HIGH
#define LED_HIGH                        1
#endif




#define DRAM_BASE	0


//----------------------------------------------------------------------------------
// defines for 21285 StrongARM Control Register

#define INIT_COMPLETE			1
#define XCS_2				0x40000000
#define XCS_1				0x20000000
#define XCS_0				0x10000000
#define ROM_TIMING			0x0fff0000
#define ROM_SETUP			0x04aa0000


 // set cycle length = 2 for all devices, strobe shift divisor = 1 (2)
 // Octal 022222
 // SuperI/O drives interrupts active high, tell 21285 to do the same

#define INIT_XBUS_CYCLE 		0x100016DB
#define INIT_XBUS_STROBE		0xfcfcfcfc

 // xcs<2> = output (softIO, leds and jumpers), xcs<1> = output (SuperIO),
 // xcs<0> = input (interrupt)
 //
#define XBUS_BITS			(XCS_2 | XCS_1 | XCS_0)
#define XBUS_INIT			XCS_2

#define SA_CTL_INIT_MASK		(XBUS_BITS | ROM_TIMING)
#define SA_CTL_INIT				(XBUS_INIT | ROM_SETUP)

#define CSR_BASE			0x42000000

// CSR register offsets 

#define MAILBOX_0			0x50
#define MAILBOX_1			0x54
#define MAILBOX_2			0x58
#define MAILBOX_3			0x5C
#define DOORBELL			0x60
#define DOORBELL_SETUP		0x64
#define ROM_BYTE_WRITE		0x68
#define CHAN_1_BYTE_COUNT	0x80
#define CHAN_1_PCI_ADDR 	0x84
#define CHAN_1_DRAM_ADDR	0x88
#define CHAN_1_DESC_PTR 	0x8C
#define CHAN_1_CONTROL		0x90
#define CHAN_2_BYTE_COUNT	0xA0
#define CHAN_2_PCI_ADDR 	0xA4
#define CHAN_2_DRAM_ADDR	0xA8
#define CHAN_2_DESC_PTR 	0xAC
#define CHAN_2_CONTROL		0xB0
#define DRAM_BASE_ADDR_MASK	0x100
#define DRAM_BASE_ADDR_OFF	0x104
#define ROM_BASE_ADDR_MASK	0x108
#define DRAM_TIMING			0x10C
#define DRAM_ADDR_SIZE_0	0x110
#define DRAM_ADDR_SIZE_1	0x114
#define DRAM_ADDR_SIZE_2	0x118
#define DRAM_ADDR_SIZE_3	0x11C
#define I2O_IFH 			0x120
#define I2O_IPT 			0x124
#define I2O_OPH 			0x128
#define I2O_OFT 			0x12C
#define I2O_IFC		 		0x130
#define I2O_OPC 			0x134
#define I2O_IPC 			0x138
#define SA_CONTROL			0x13C
#define PCI_ADDR_EXT		0x140
#define PREFETCH_RANGE		0x144
#define XBUS_CYCLE			0x148
#define XBUS_STROBE			0x14C
#define DBELL_PCI_MASK		0x150
#define DBELL_SA_MASK		0x154
#define AMBA_SERIAL_BASE	(CSR_BASE + 0x160)


//---------------------------------------------------------------------------
// The following are used in the startup code for the EBSA-285. There are two
// DIMMs of RAM on the EBSA-285, and there may be DRAM in either block. The
// DRAM is initialised by the startup code.

// MMU
#define CP_MMU				15


#define MMUCR_M_ENABLE	(1)	/* MMU enable */
#define MMUCR_A_ENABLE	(1<<1)	/* Address alignment fault enable */
#define MMUCR_C_ENABLE	(1<<2)	/* (data) cache enable */
#define MMUCR_W_ENABLE	(1<<3)	/* write buffer enable */
#define MMUCR_PROG32	(1<<4)	/* PROG32 */
#define MMUCR_DATA32	(1<<5)	/* DATA32 */
#define MMUCR_L_ENABLE	(1<<6)	/* Late abort on earlier CPUs */
#define MMUCR_BIGEND	(1<<7)	/* Big-endian (=1), little-endian (=0) */
#define MMUCR_SYSTEM	(1<<8)	/* System bit, modifies MMU protections */
#define MMUCR_ROM		(1<<9)	/* ROM bit, modifies MMU protections */
#define MMUCR_F			(1<<10)	/* Should Be Zero */
#define MMUCR_Z_ENABLE	(1<<11)	/* Branch prediction enable on 810 */
#define MMUCR_I_ENABLE	(1<<12)	/* Instruction cache enable on SA110 */

#define MMU_INIT_VALUE (MMUCR_PROG32 | MMUCR_DATA32 | MMUCR_L_ENABLE | MMUCR_ROM)


#define MAX_SDRAM			4
// DC21285 Addresses 

#define DC21285_DRAM_A0MR	0x40000000
#define DC21285_DRAM_A1MR	0x40004000
#define DC21285_DRAM_A2MR	0x40008000
#define DC21285_DRAM_A3MR	0x4000C000
// SD_OFFSET is the difference between SDRAM Array Register banks 
#define SD_OFFSET		(DC21285_DRAM_A1MR - DC21285_DRAM_A0MR)
// Row/Col Mux modes shifted into correct postion (bits 6, 5 and 4)
#define MUX_MODE0		0x00
#define MUX_MODE1		0x10
#define MUX_MODE2		0x20
#define MUX_MODE3		0x30
#define MUX_MODE4		0x40

// Bit patterns for SDRAM memory sizes 
#define SDSIZE_1M		0x01
#define SDSIZE_2M		0x02
#define SDSIZE_4M		0x03
#define SDSIZE_8M		0x04
#define SDSIZE_16M		0x05
#define SDSIZE_32M		0x06
#define SDSIZE_64M		0x07
// 8MB bit pattern for EBSA-285 base memory: 1*16 MB DIMM: 2 * 8 MB arrays 

#define EBSA_BASE_MEM		(SDSIZE_8M | MUX_MODE1)
#define EBSA_BASE_SIZE		0x00800000
#define CAS_OFFSET			0x88					//  If CAS Latency is 2 cycles 
// See 21285 Page 7-31 of data sheet: 
#define DC21285_Trp1		0x0
#define DC21285_Trp2		0x1
#define DC21285_Trp3		0x2
#define DC21285_Trp4		0x3
#define DC21285_Tdal2		(0x0 << 2)
#define DC21285_Tdal3		(0x1 << 2)
#define DC21285_Tdal4		(0x2 << 2)
#define DC21285_Tdal5		(0x3 << 2)
#define DC21285_Trcd2		(0x2 << 4)
#define DC21285_Trcd3		(0x3 << 4)
#define DC21285_Tcas2		(0x2 << 6)
#define DC21285_Tcas3		(0x3 << 6)
#define DC21285_Trc4		(0x1 << 8)
#define DC21285_Trc5		(0x2 << 8)
#define DC21285_Trc6		(0x3 << 8)
#define DC21285_Trc7		(0x4 << 8)
#define DC21285_Trc8		(0x5 << 8)
#define DC21285_Trc9		(0x6 << 8)
#define DC21285_Trc10		(0x7 << 8)
#define DC21285_Tcd0		(0x0 << 11)
#define DC21285_Tcd1		(0x1 << 11)
#define DC21285_Parity0 	(0x0 << 12)
#define DC21285_Parity1 	(0x1 << 12)

 //
 // Tref, No Parity, CDT=1, Trc=6, Tcas=2, Trcd=2, Tdal(Trp+Trdl)=3, Trp=2
 //	0	   1	  011	 10	 10	 01		   01
 //
#define saTrp				DC21285_Trp2
#define saTdal				DC21285_Tdal3
#define saTrcd				DC21285_Trcd2
#define saTcas				DC21285_Tcas2
#define saTrc				DC21285_Trc6
#define saTcd				DC21285_Tcd1
#define saParity			DC21285_Parity0

#define SDRAM_MODEBITS		(saParity + saTcd + saTrc + saTcas + \
				 saTrcd + saTdal + saTrp)

// Tref=1 

#define INITIAL_TREF		(0x01 << 16)	 //  Top half of word 
#define INITIAL_TIMING		(INITIAL_TREF + SDRAM_MODEBITS)
// Tref=1a (64 ms x 4096 rows) = 15.xx us 
#define SETUP_TREF			(0x1a << 16)	 //  Top half of word 
#define SETUP_TIMING		(SETUP_TREF + SDRAM_MODEBITS)

#define REFRESH_COUNT		32				// No. of cycles scaled per wait 
#define REFRESH_WAIT		8				// Max. wait per SDRAM 


//------------------------------------------------------------------------------
// definitions for the AMBA UART in the 21285 chip

#define UART_XTAL_FREQ			(50000000/4)	// UART baud rate clock freq 
#define N_UART_CHANNELS			1				// total number UART channels
#define N_SIO_CHANNELS			1				// total num of SIO channels 
#define N_AMBA_UART_CHANNELS	1				// number AMBA UART channels 


 // Definitions for the AMBA Timer:
 // There are four timers, three clocked from the same source and one clocked
 //from a 3.6864 MHz oscillator (timer 3). Timer 4 may be used as a watchdog,
 // so we choose timer 2 for the aux clock, and timer 3 for the system timer.
 //

#define AMBA_TIMER_BASE		0x42000300	// Address of base of timer 


 // Run the aux clock from system clock (not pre-divided) and the sys clock from
 // the external reference clock.

#define AMBA_TIMER_SYS_TC_DISABLE	(TC_DISABLE | TC_PERIODIC | TC_EXTERNAL)
#define AMBA_TIMER_SYS_TC_ENABLE	(TC_ENABLE  | TC_PERIODIC | TC_EXTERNAL)
#define AMBA_TIMER_AUX_TC_DISABLE	(TC_DISABLE | TC_PERIODIC)
#define AMBA_TIMER_AUX_TC_ENABLE	(TC_ENABLE  | TC_PERIODIC)

#define SYS_TIMER_CLK	 	3686400		// Frequency of counter/timer
#define AUX_TIMER_CLK		50000000	// Frequency of counter/timer

#define AMBA_RELOAD_TICKS	0			// no overhead reloading timer
#define SYS_TIMER_CLEAR(x)	(AMBA_TIMER_T3CLEAR(x)) // sys Clk is timer 3 
#define SYS_TIMER_CTRL(x)	(AMBA_TIMER_T3CTRL(x))
#define SYS_TIMER_LOAD(x)	(AMBA_TIMER_T3LOAD(x))
#define SYS_TIMER_VALUE(x)	(AMBA_TIMER_T3VALUE(x))
#define AUX_TIMER_CLEAR(x)	(AMBA_TIMER_T2CLEAR(x)) // aux Clk is timer 2 
#define AUX_TIMER_CTRL(x)	(AMBA_TIMER_T2CTRL(x))
#define AUX_TIMER_LOAD(x)	(AMBA_TIMER_T2LOAD(x))
#define AUX_TIMER_VALUE(x)	(AMBA_TIMER_T2VALUE(x))

#define SYS_TIMER_INT_LVL 	(INT_LVL_TIMER_3)
#define AUX_TIMER_INT_LVL	(INT_LVL_TIMER_2)


 // Maximum clock rates depend upon CPU power and work load of application.
 // They will also critically depend on whether caches are enabled.
 // The values below are minimum and maximum allowed by the hardware/software.
 // The counter is 24-bit and we don't accept values that would mean a zero
 // reload value as we don't know what that will do.
 
#define SYS_CLK_RATE_MIN ((SYS_TIMER_CLK+0xFFFFFF)/0x1000000)
#define AUX_CLK_RATE_MIN ((AUX_TIMER_CLK+0xFFFFFF)/0x1000000)
#define SYS_CLK_RATE_MAX 250000
#define AUX_CLK_RATE_MAX 250000

#define PORT_COM1		0x3f8
#define PORT_COM2		0x2f8
#define IO_SHIFT		2

#define HOST_COMPORT	AMBA_SERIAL_BASE
#define OS_COMPORT		PORT_COM2

#define AMBA_UARTDR     0x00
#define AMBA_RXSTAT     0x04
#define AMBA_H_UBRLCR   0x08
#define AMBA_M_UBRLCR   0x0C
#define AMBA_L_UBRLCR   0x10
#define AMBA_UARTCON    0x14
#define AMBA_UARTFLG    0x18

	
#define AMBA_UARTFLG_TFF 	               	0x20
#define AMBA_UARTFLG_RFE    	            0x10
#define AMBA_UARTFLG_TBY        	        0x08
#define AMBA_UARTFLG_TMSK           	    (AMBA_UARTFLG_TFF + AMBA_UARTFLG_TBY)
	
#define AMBA_UARTCON_I                 		0x04
#define AMBA_UARTCON_S                 		0x02
#define AMBA_UARTCON_UE                		0x01

#define AMBA_H_UBRLCR_DSS_8                 0x60
#define AMBA_H_UBRLCR_DSS_7                 0x40
#define AMBA_H_UBRLCR_DSS_6                 0x20
#define AMBA_H_UBRLCR_DSS_5                 0x00
#define AMBA_H_UBRLCR_EF                    0x10
#define AMBA_H_UBRLCR_SBS                   0x08
#define AMBA_H_UBRLCR_OES                   0x04
#define AMBA_H_UBRLCR_PE                    0x02
#define AMBA_H_UBRLCR_BRK                   0x01

//--------------------------------------------------------------------------------------
// Flash configuration
 
#ifdef INCLUDE_FLASH
#define FLASH_WIDTH		4	// 32-bit wide 

#define SYS_FLASH_BOARD_DELAY
#define SYS_FLASH_DELAY_SHIFT 1
#define SYS_FLASH_DELAY_ADJ 0
#define SYS_FLASH_DELAY_INCR 1
#endif // INCLUDE_FLASH


 // PCI definitions
 // 
 // First, defines for generic pciIoMapLib.c code
 // 
 // Cache Line Size - in number of 32-bit words
 // It might be thought that we would set this to (_CACHE_ALIGN_SIZE/4), but,
 // in fact, the areas of memory mapped for use either by PCI I/O and
 // PCI Configuration  and the area of (ARM) memory used by the Ethernet
 // controller are marked as non-cacheable, so we should set it to zero.

#define PCI_CLINE_SZ	0

 // Latency Timer value
 // A sensible value for this depends what we are using PCI for.
 // (Rather arbitrarily) leave it at 0 (the reset default value)

#define PCI_LAT_TIMER           0x0

#define PCI_IN_BYTE(x)		*(UINT8 *) (x)
#define PCI_IN_WORD(x)		sysRead16((UINT16 *)(x))
#define PCI_IN_LONG(x)		*(UINT32 *)(x)
#define PCI_OUT_BYTE(x,y)	*(UINT8 *) (x) = (UINT8)  y
#define PCI_OUT_WORD(x,y)	sysWrite16((UINT16 *)(x), (UINT16)(y))
#define PCI_OUT_LONG(x,y)	*(UINT32 *)(x) = (UINT32) y


 // Dummy interrupt number for pciIntConnect(): must be passed a real
 // interrupt vector number. This allows for non-linear mappings
 // between interrupts, slots etc.
 
#define INT_NUM_IRQ0	0

// memory map as seen by the CPU on the local bus 
#define CPU_PCI_IO_ADRS		0x7C000000	// PCI I/O space base 
#define CPU_PCI_IO_SIZE		0x00010000	// 64 kbytes 
#define CPU_PCI_CNFG_ADRS	0x7B000000	// type 0 PCI config space 
#define CPU_PCI_CNFG_SIZE	0x01000000	// 16 Mbytes 

#define CPU_PCI_CNFG_1_ADRS	0x7A000000	// type 1 PCI config space 
#define CPU_PCI_CNFG_1_SIZE	0x01000000	// 16 Mbytes 

// PCI view of PCI I/O space for PCI devices 
#define PCI_IO_ADRS		0x00000000		// base of PCI I/O address 
#define PCI2DRAM_BASE_ADRS	0x00000000	// memory seen from PCI bus

// PCI bus resources 
#define PCI_IO_SIZE		0x80            // PCI I/O size 

// PCI device 0: slot I/O 4 
#define PCI_MEM_ADR0            0x00            // Memory base for CSR 
#define PCI_IO_ADR0             PCI_IO_ADRS     // I/O base for CSR 32Bytes 
#define PCI_IO_SIZE0            PCI_IO_SIZE     // I/O size for CSR 
#define PCI_INT_LVL0            INT_LVL_PCI		// Interrupt level 
#define PCI_INT_VEC0            IVEC_TO_INUM(PCI_INT_LVL0) // Interrupt vector 

// PCI device 1: slot I/O 3 
#define PCI_MEM_ADR1            0x00
#define PCI_IO_ADR1             PCI_IO_ADRS
#define PCI_IO_SIZE1            PCI_IO_SIZE
#define PCI_INT_LVL1            INT_LVL_IRQ_IN_3
#define PCI_INT_VEC1            IVEC_TO_INUM(PCI_INT_LVL1)

// PCI device 2: slot I/O 2 
#define PCI_MEM_ADR2            0x00
#define PCI_IO_ADR2             PCI_IO_ADRS
#define PCI_IO_SIZE2            PCI_IO_SIZE
#define PCI_INT_LVL2            INT_LVL_IRQ_IN_1
#define PCI_INT_VEC2            IVEC_TO_INUM(PCI_INT_LVL2)

// PCI device 3: slot I/O 1
#define PCI_MEM_ADR3            0x00
#define PCI_IO_ADR3             PCI_IO_ADRS
#define PCI_IO_SIZE3            PCI_IO_SIZE
#define PCI_INT_LVL3            INT_LVL_IRQ_IN_0
#define PCI_INT_VEC3            IVEC_TO_INUM(PCI_INT_LVL3)

 // N.B. Only the following values have been tested. In particular, no
 // testing has been performed with more than one PCI bus, i.e. Type 1
 // configuration.

#define PCI_MAX_DEV		4		// 4 PCI slots: 0 to 3 
#define PCI_MAX_BUS		1		// Only one bus 

// Allocate Device Space (Offsets) for PCI devices 
#define LAN_DEV_SPACE		0
#define PCI_IO_LN_ADRS		(PCI_IO_ADRS + LAN_DEV_SPACE)

// Allocated base address of h/w devices as seen from CPU 
#define LAN_BASE_ADRS		(CPU_PCI_IO_ADRS + LAN_DEV_SPACE)

// Create configuration address 
#define MAKE_CFG_ADDR(bus, device, func, offset) ((bus) ? \
    ((UINT32)((bus & 0xFF) << 16) | (UINT32)((device & 0x1F) << 11) | \
     (UINT32)((func & 0x7) << 8)  | 0x01) : \
    ((UINT32)(1 << ((device & 0x1F) + 16)) | (UINT32)((func & 0x7) << 8) | \
     (UINT32) (offset & 0xFC)))




#define NoIRQ		0x80			// Bit 7 of cspr
#define NoFIQ		0x40			// Bit 6 of cspr
#define NoINTS		(NoIRQ | NoFIQ) //
#define MaskINTS	NoINTS
     
#endif // end #ifdef include_ebsa285_h
